/*
 PKM_Delta.h - Test library for Delta Robot Inverse Kinematics
 Copyright (c) 2009 Torsten Voigt, Inverse Kinemtics based on PHD Thesis by R.Stamper
 All right reserved.
 */





// include this library's description file
#include "PKM_Delta.h"

// include description files for other libraries used (if any)
#include "HardwareSerial.h"

// Constructor /////////////////////////////////////////////////////////////////
// Function that handles the creation and setup of instances

PKM_Delta::PKM_Delta(double base_radius,double thigh_length,double tibia_length,double moving_base_radius,double spacing_angle_1,double spacing_angle_2)
{
  // initialize this instance's variables
	 _r=base_radius;
	 _a=thigh_length;
	 _b=tibia_length;			
	 _c=moving_base_radius;
	
	
	 cos_teta0[0]=cos(0);							//initialize the constants for the coordinate transformation
	 sin_teta0[0]=sin(0);
	 cos_teta0[1]=cos(spacing_angle_1*PI/180);
	 sin_teta0[1]=sin(spacing_angle_1*PI/180);
	 cos_teta0[2]=cos(spacing_angle_2*PI/180);
	 sin_teta0[2]=sin(spacing_angle_2*PI/180);
	
 
}

// Public Methods //////////////////////////////////////////////////////////////
// Functions available in Wiring sketches, this library, and other libraries

int PKM_Delta::get_leg_angles(double *pt, double *teta1)
  {
	  for (int _i=0;_i<3;_i++)	
	{
		double pu = pt[0]*cos_teta0[_i] + pt[1]*sin_teta0[_i] - _r;			// transform xyz coordinates into uvw workspace for individual leg
		double pv = -pt[0]*sin_teta0[_i] + pt[1]*cos_teta0[_i];
		double pw = pt[2];
		
		/* If point out of the workspace */
		
		double pv_b=pv/_b;												
		
		if (abs(pv_b) > 1.0 )
		{
			return -1.0;												// check for points out of workspace, check for leg relative v coordinate beeing larger than thigh_length
		}
		else
		{
			double aacc=_a*_a+_c*_c;
			double _2ac=2*_a*_c;
			double _2apu=2*_a*pu;
			
			double l0 = pu*pu + pw*pw + 2*_c*pu - _2apu- _b*_b*(1-pv_b*pv_b) ;	 // sin2 (teta3) = 1-cos2(teta3); cos(teta3)=pv/b=pv_b */
			double l2 = l0+2*_2apu;
			l0 = l0 - _2ac;
			l0 = l0 + aacc;
			double l1 = -4*_a*pw;
			l2 = l2+ _2ac;
			l2 = l2 + aacc;
			
			double _2l2=2*l2;
			double discr=l1*l1-2*_2l2*l0;
			if (discr < 0)														// check if there is a real solution to the equation set
			{
				return -1.0;
			}
			else
			{
				
				
				double first_addend=(-l1)/(_2l2);
				double second_addend=(sqrt(discr))/(_2l2);
				double sum1=first_addend+second_addend;
				double sum2=first_addend-second_addend;
				if (abs(sum1)<abs(sum1))
				{
					teta1[_i] = 2*atan(sum1);											// check if approx is quicker
				}
				else
				{
					teta1[_i] = 2*atan(sum2);
				}
				
				return 0;
			}
		}	
	}
  }
  // it can also call private functions of this library
 
// Private Methods /////////////////////////////////////////////////////////////
// Functions only available to other functions in this library

